@@ -150,7 +150,7 @@ func New(docCall bool) *cobra.Command {
 
 	rootCommand.PersistentFlags().BoolVarP(&headless, "headless", "", false, "Run debug server only, in headless mode. Server will accept both JSON-RPC or DAP client connections.")
 	rootCommand.PersistentFlags().BoolVarP(&acceptMulti, "accept-multiclient", "", false, "Allows a headless server to accept multiple client connections via JSON-RPC or DAP.")
-	rootCommand.PersistentFlags().IntVar(&apiVersion, "api-version", 1, "Selects JSON-RPC API version when headless. New clients should use v2. Can be reset via RPCServer.SetApiVersion. See Documentation/api/json-rpc/README.md.")
+	rootCommand.PersistentFlags().IntVar(&apiVersion, "api-version", 2, "Selects JSON-RPC API version when headless. The only valid value is 2. Can be reset via RPCServer.SetApiVersion. See Documentation/api/json-rpc/README.md.")
 	must(rootCommand.RegisterFlagCompletionFunc("api-version", cobra.FixedCompletions([]string{"1", "2"}, cobra.ShellCompDirectiveNoFileComp)))
 	rootCommand.PersistentFlags().StringVar(&initFile, "init", "", "Init file, executed by the terminal client.")
 	must(rootCommand.MarkPersistentFlagFilename("init"))
@@ -244,7 +244,7 @@ will exit when the debug session ends.`,
 		Run:               dapCmd,
 		ValidArgsFunction: cobra.NoFileCompletions,
 	}
-	dapCommand.Flags().StringVar(&dapClientAddr, "client-addr", "", "host:port where the DAP client is waiting for the DAP server to dial in")
+	dapCommand.Flags().StringVar(&dapClientAddr, "client-addr", "", "Address where the DAP client is waiting for the DAP server to dial in. Prefix with 'unix:' to use a unix domain socket.")
 	must(dapCommand.RegisterFlagCompletionFunc("client-addr", cobra.NoFileCompletions))
 
 	// TODO(polina): support --tty when dlv dap allows to launch a program from command-line
@@ -463,7 +463,8 @@ Some backends can be configured using environment variables:
 * DELVE_DEBUGSERVER_PATH specifies the path of the debugserver executable for the lldb backend
 * DELVE_RR_RECORD_FLAGS specifies additional flags used when calling 'rr record'
 * DELVE_RR_REPLAY_FLAGS specifies additional flags used when calling 'rr replay'
-`})
+`,
+	})
 
 	rootCommand.AddCommand(&cobra.Command{
 		Use:   "log",
@@ -512,6 +513,24 @@ File redirects can also be changed using the 'restart' command.
 `,
 	})
 
+	rootCommand.AddCommand(&cobra.Command{
+		Use:    "substitute-path-guess-helper",
+		Hidden: true,
+		Run: func(cmd *cobra.Command, args []string) {
+			gsp, err := rpc2.MakeGuessSusbtitutePathIn()
+			if err != nil {
+				fmt.Printf("ERROR: %v\n", err)
+				os.Exit(1)
+			}
+			err = json.NewEncoder(os.Stdout).Encode(gsp)
+			if err != nil {
+				fmt.Printf("ERROR: %v\n", err)
+				os.Exit(1)
+			}
+			os.Exit(0)
+		},
+	})
+
 	rootCommand.DisableAutoGenTag = true
 
 	configUsageFunc(rootCommand)
@@ -581,12 +600,7 @@ func dapCmd(cmd *cobra.Command, args []string) {
 			}
 			cfg.Listener = listener
 		} else { // with a predetermined client.
-			var err error
-			conn, err = net.Dial("tcp", dapClientAddr)
-			if err != nil {
-				fmt.Fprintf(os.Stderr, "Failed to connect to the DAP client: %v\n", err)
-				return 1
-			}
+			conn = netDial(dapClientAddr)
 		}
 
 		server := dap.NewServer(cfg)
