@@ -2,6 +2,7 @@ package goversion
 
 import (
 	"fmt"
+	"runtime"
 
 	"github.com/go-delve/delve/pkg/logflags"
 )
@@ -10,7 +11,7 @@ var (
 	MinSupportedVersionOfGoMajor = 1
 	MinSupportedVersionOfGoMinor = 21
 	MaxSupportedVersionOfGoMajor = 1
-	MaxSupportedVersionOfGoMinor = 23
+	MaxSupportedVersionOfGoMinor = 24
 	goTooOldErr                  = fmt.Sprintf("Go version %%s is too old for this version of Delve (minimum supported version %d.%d, suppress this error with --check-go-version=false)", MinSupportedVersionOfGoMajor, MinSupportedVersionOfGoMinor)
 	goTooOldWarn                 = fmt.Sprintf("WARNING: undefined behavior - Go version %%s is too old for this version of Delve (minimum supported version %d.%d)", MinSupportedVersionOfGoMajor, MinSupportedVersionOfGoMinor)
 	dlvTooOldErr                 = fmt.Sprintf("Version of Delve is too old for Go version %%s (maximum supported version %d.%d, suppress this error with --check-go-version=false)", MaxSupportedVersionOfGoMajor, MaxSupportedVersionOfGoMinor)
@@ -21,9 +22,16 @@ var (
 // this version of delve.
 func Compatible(producer string, warnonly bool) error {
 	ver := ParseProducer(producer)
-	if ver.IsDevel() {
+	if ver.IsOldDevel() {
 		return nil
 	}
+	if runtime.GOARCH == "riscv64" && !ver.AfterOrEqual(GoVersion{1, 24, versionedDevel, "", ""}) {
+		if warnonly {
+			logflags.WriteError(fmt.Sprintf(goTooOldWarn, ver.String()))
+			return nil
+		}
+		return fmt.Errorf(goTooOldErr, ver.String())
+	}
 	if !ver.AfterOrEqual(GoVersion{MinSupportedVersionOfGoMajor, MinSupportedVersionOfGoMinor, betaRev(0), "", ""}) {
 		if warnonly {
 			logflags.WriteError(fmt.Sprintf(goTooOldWarn, ver.String()))
