@@ -252,7 +252,7 @@ func (t *Target) Valid() (bool, error) {
 // Currently only non-recorded processes running on AMD64 support
 // function calls.
 func (t *Target) SupportsFunctionCalls() bool {
-	return t.Process.BinInfo().Arch.Name == "amd64" || (t.Process.BinInfo().Arch.Name == "arm64" && t.Process.BinInfo().GOOS != "windows")
+	return t.Process.BinInfo().Arch.Name == "amd64" || (t.Process.BinInfo().Arch.Name == "arm64" && t.Process.BinInfo().GOOS != "windows") || t.Process.BinInfo().Arch.Name == "ppc64le"
 }
 
 // ClearCaches clears internal caches that should not survive a restart.
@@ -260,6 +260,7 @@ func (t *Target) SupportsFunctionCalls() bool {
 func (t *Target) ClearCaches() {
 	t.clearFakeMemory()
 	t.gcache.Clear()
+	t.BinInfo().moduleDataCache = nil
 	for _, thread := range t.ThreadList() {
 		thread.Common().g = nil
 	}
@@ -296,55 +297,33 @@ func (t *Target) SelectedGoroutine() *G {
 }
 
 // SwitchGoroutine will change the selected and active goroutine.
-func (p *Target) SwitchGoroutine(g *G) error {
-	if ok, err := p.Valid(); !ok {
+func (t *Target) SwitchGoroutine(g *G) error {
+	if ok, err := t.Valid(); !ok {
 		return err
 	}
 	if g == nil {
 		return nil
 	}
 	if g.Thread != nil {
-		return p.SwitchThread(g.Thread.ThreadID())
+		return t.SwitchThread(g.Thread.ThreadID())
 	}
-	p.selectedGoroutine = g
+	t.selectedGoroutine = g
 	return nil
 }
 
 // SwitchThread will change the selected and active thread.
-func (p *Target) SwitchThread(tid int) error {
-	if ok, err := p.Valid(); !ok {
+func (t *Target) SwitchThread(tid int) error {
+	if ok, err := t.Valid(); !ok {
 		return err
 	}
-	if th, ok := p.FindThread(tid); ok {
-		p.currentThread = th
-		p.selectedGoroutine, _ = GetG(p.CurrentThread())
+	if th, ok := t.FindThread(tid); ok {
+		t.currentThread = th
+		t.selectedGoroutine, _ = GetG(t.CurrentThread())
 		return nil
 	}
 	return fmt.Errorf("thread %d does not exist", tid)
 }
 
-// detach will detach the target from the underlying process.
-// This means the debugger will no longer receive events from the process
-// we were previously debugging.
-// If kill is true then the process will be killed when we detach.
-func (t *Target) detach(kill bool) error {
-	if !kill {
-		if t.asyncPreemptChanged {
-			setAsyncPreemptOff(t, t.asyncPreemptOff)
-		}
-		for _, bp := range t.Breakpoints().M {
-			if bp != nil {
-				err := t.ClearBreakpoint(bp.Addr)
-				if err != nil {
-					return err
-				}
-			}
-		}
-	}
-	t.StopReason = StopUnknown
-	return t.proc.Detach(kill)
-}
-
 // setAsyncPreemptOff enables or disables async goroutine preemption by
 // writing the value 'v' to runtime.debug.asyncpreemptoff.
 // A value of '1' means off, a value of '0' means on.
@@ -356,8 +335,12 @@ func setAsyncPreemptOff(p *Target, v int64) {
 	scope := globalScope(p, p.BinInfo(), p.BinInfo().Images[0], p.Memory())
 	// +rtype -var debug anytype
 	debugv, err := scope.findGlobal("runtime", "debug")
-	if err != nil || debugv.Unreadable != nil {
-		logger.Warnf("could not find runtime/debug variable (or unreadable): %v %v", err, debugv.Unreadable)
+	if err != nil {
+		logger.Warnf("could not find runtime/debug variable (or unreadable): %v", err)
+		return
+	}
+	if debugv.Unreadable != nil {
+		logger.Warnf("runtime/debug variable unreadable: %v", err, debugv.Unreadable)
 		return
 	}
 	asyncpreemptoffv, err := debugv.structMember("asyncpreemptoff") // +rtype int32
@@ -396,20 +379,18 @@ func (t *Target) createUnrecoveredPanicBreakpoint() {
 
 // createFatalThrowBreakpoint creates the a breakpoint as runtime.fatalthrow.
 func (t *Target) createFatalThrowBreakpoint() {
-	fatalpcs, err := FindFunctionLocation(t.Process, "runtime.throw", 0)
-	if err == nil {
-		bp, err := t.SetBreakpoint(fatalThrowID, fatalpcs[0], UserBreakpoint, nil)
+	setFatalThrow := func(pcs []uint64, err error) {
 		if err == nil {
-			bp.Logical.Name = FatalThrow
-		}
-	}
-	fatalpcs, err = FindFunctionLocation(t.Process, "runtime.fatal", 0)
-	if err == nil {
-		bp, err := t.SetBreakpoint(fatalThrowID, fatalpcs[0], UserBreakpoint, nil)
-		if err == nil {
-			bp.Logical.Name = FatalThrow
+			bp, err := t.SetBreakpoint(fatalThrowID, pcs[0], UserBreakpoint, nil)
+			if err == nil {
+				bp.Logical.Name = FatalThrow
+			}
 		}
 	}
+	setFatalThrow(FindFunctionLocation(t.Process, "runtime.throw", 0))
+	setFatalThrow(FindFunctionLocation(t.Process, "runtime.fatal", 0))
+	setFatalThrow(FindFunctionLocation(t.Process, "runtime.winthrow", 0))
+	setFatalThrow(FindFunctionLocation(t.Process, "runtime.fatalsignal", 0))
 }
 
 // createPluginOpenBreakpoint creates a breakpoint at the return instruction
@@ -453,6 +434,11 @@ func (t *Target) GetBufferedTracepoints() []*UProbeTraceResult {
 		v.Addr = ip.Addr
 		v.Kind = ip.Kind
 
+		if v.RealType == nil {
+			v.Unreadable = errors.New("type not supported by ebpf")
+			return v
+		}
+
 		cachedMem := CreateLoadedCachedMemory(ip.Data)
 		compMem, _ := CreateCompositeMemory(cachedMem, t.BinInfo().Arch, op.DwarfRegisters{}, ip.Pieces, ip.RealType.Common().ByteSize)
 		v.mem = compMem
@@ -483,16 +469,16 @@ func (t *Target) GetBufferedTracepoints() []*UProbeTraceResult {
 
 // ResumeNotify specifies a channel that will be closed the next time
 // Continue finishes resuming the targets.
-func (t *TargetGroup) ResumeNotify(ch chan<- struct{}) {
-	t.cctx.ResumeChan = ch
+func (grp *TargetGroup) ResumeNotify(ch chan<- struct{}) {
+	grp.cctx.ResumeChan = ch
 }
 
 // RequestManualStop attempts to stop all the processes' threads.
-func (t *TargetGroup) RequestManualStop() error {
-	t.cctx.StopMu.Lock()
-	defer t.cctx.StopMu.Unlock()
-	t.cctx.manualStopRequested = true
-	return t.Selected.proc.RequestManualStop(t.cctx)
+func (grp *TargetGroup) RequestManualStop() error {
+	grp.cctx.StopMu.Lock()
+	defer grp.cctx.StopMu.Unlock()
+	grp.cctx.manualStopRequested = true
+	return grp.Selected.proc.RequestManualStop(grp.cctx)
 }
 
 const (
@@ -651,3 +637,9 @@ func (*dummyRecordingManipulation) ClearCheckpoint(int) error { return ErrNotRec
 func (*dummyRecordingManipulation) Restart(*ContinueOnceContext, string) (Thread, error) {
 	return nil, ErrNotRecorded
 }
+
+var ErrWaitForNotImplemented = errors.New("waitfor not implemented")
+
+func (waitFor *WaitFor) Valid() bool {
+	return waitFor != nil && waitFor.Name != ""
+}
