@@ -27,7 +27,6 @@ type Thread interface {
 	BinInfo() *BinaryInfo
 	// ProcessMemory returns the process memory.
 	ProcessMemory() MemoryReadWriter
-	StepInstruction() error
 	// SetCurrentBreakpoint updates the current breakpoint of this thread, if adjustPC is true also checks for breakpoints that were just hit (this should only be passed true after a thread resume)
 	SetCurrentBreakpoint(adjustPC bool) error
 	// SoftExc returns true if this thread received a software exception during the last resume.
@@ -67,14 +66,14 @@ func (t *CommonThread) ReturnValues(cfg LoadConfig) []*Variable {
 }
 
 // topframe returns the two topmost frames of g, or thread if g is nil.
-func topframe(g *G, thread Thread) (Stackframe, Stackframe, error) {
+func topframe(tgt *Target, g *G, thread Thread) (Stackframe, Stackframe, error) {
 	var frames []Stackframe
 	var err error
 
 	if g == nil {
-		frames, err = ThreadStacktrace(thread, 1)
+		frames, err = ThreadStacktrace(tgt, thread, 1)
 	} else {
-		frames, err = g.Stacktrace(1, StacktraceReadDefers)
+		frames, err = GoroutineStacktrace(tgt, g, 1, StacktraceReadDefers)
 	}
 	if err != nil {
 		return Stackframe{}, Stackframe{}, err
