@@ -15,10 +15,10 @@ import (
 // some extra flags defined here.
 // See equivalent declaration in $GOROOT/src/reflect/type.go
 const (
-	kindDirectIface = 1 << 5 // +rtype kindDirectIface
-	kindGCProg      = 1 << 6 // +rtype kindGCProg
+	kindDirectIface = 1 << 5 // +rtype kindDirectIface|internal/abi.KindDirectIface
+	kindGCProg      = 1 << 6 // +rtype kindGCProg|internal/abi.KindGCProg
 	kindNoPointers  = 1 << 7
-	kindMask        = (1 << 5) - 1 // +rtype kindMask
+	kindMask        = (1 << 5) - 1 // +rtype kindMask|internal/abi.KindMask
 )
 
 type runtimeTypeDIE struct {
@@ -89,7 +89,7 @@ func (ctxt *loadDebugInfoMapsContext) lookupAbstractOrigin(bi *BinaryInfo, off d
 	return r
 }
 
-// runtimeTypeToDIE returns the DIE corresponding to the runtime._type.
+// RuntimeTypeToDIE returns the DIE corresponding to the runtime._type.
 // This is done in three different ways depending on the version of go.
 //   - Before go1.7 the type name is retrieved directly from the runtime._type
 //     and looked up in debug_info
@@ -98,18 +98,13 @@ func (ctxt *loadDebugInfoMapsContext) lookupAbstractOrigin(bi *BinaryInfo, off d
 //     debug_info
 //   - After go1.11 the runtimeTypeToDIE map is used to look up the address of
 //     the type and map it directly to a DIE.
-func runtimeTypeToDIE(_type *Variable, dataAddr uint64) (typ godwarf.Type, kind int64, err error) {
+func RuntimeTypeToDIE(_type *Variable, dataAddr uint64, mds []ModuleData) (typ godwarf.Type, kind int64, err error) {
 	bi := _type.bi
 
 	_type = _type.maybeDereference()
 
 	// go 1.11 implementation: use extended attribute in debug_info
 
-	mds, err := loadModuleData(bi, _type.mem)
-	if err != nil {
-		return nil, 0, fmt.Errorf("error loading module data: %v", err)
-	}
-
 	md := findModuleDataForType(bi, mds, _type.Addr, _type.mem)
 	if md != nil {
 		so := bi.moduleDataToImage(md)
@@ -131,7 +126,7 @@ func runtimeTypeToDIE(_type *Variable, dataAddr uint64) (typ godwarf.Type, kind
 		}
 	}
 
-	return nil, 0, fmt.Errorf("could not resolve interface type")
+	return nil, 0, errors.New("could not resolve interface type")
 }
 
 // resolveParametricType returns the real type of t if t is a parametric
@@ -154,7 +149,18 @@ func resolveParametricType(bi *BinaryInfo, mem MemoryReadWriter, t godwarf.Type,
 	}
 	_type := newVariable("", rtypeAddr, runtimeType, bi, mem)
 
-	typ, _, err := runtimeTypeToDIE(_type, 0)
+	var mds []ModuleData
+	if bi.moduleDataCache != nil {
+		mds = bi.moduleDataCache
+	} else {
+		mds, err = LoadModuleData(bi, _type.mem)
+		if err != nil {
+			return ptyp.TypedefType.Type, fmt.Errorf("error loading module data: %v", err)
+		}
+		bi.moduleDataCache = mds
+	}
+
+	typ, _, err := RuntimeTypeToDIE(_type, 0, mds)
 	if err != nil {
 		return ptyp.TypedefType.Type, err
 	}
@@ -175,7 +181,7 @@ func dwarfToRuntimeType(bi *BinaryInfo, mem MemoryReadWriter, typ godwarf.Type)
 		return 0, 0, false, nil
 	}
 
-	mds, err := loadModuleData(bi, mem)
+	mds, err := LoadModuleData(bi, mem)
 	if err != nil {
 		return 0, 0, false, err
 	}
@@ -200,7 +206,10 @@ func dwarfToRuntimeType(bi *BinaryInfo, mem MemoryReadWriter, typ godwarf.Type)
 	if kindv == nil || kindv.Unreadable != nil || kindv.Kind != reflect.Uint {
 		kindv = _type.loadFieldNamed("Kind_")
 	}
-	if kindv == nil || kindv.Unreadable != nil || kindv.Kind != reflect.Uint {
+	if kindv == nil {
+		return 0, 0, false, fmt.Errorf("unreadable interface type (no kind field)")
+	}
+	if kindv.Unreadable != nil || kindv.Kind != reflect.Uint {
 		return 0, 0, false, fmt.Errorf("unreadable interface type: %v", kindv.Unreadable)
 	}
 	typeKind, _ = constant.Uint64Val(kindv.Value)
