@@ -3,6 +3,7 @@ package terminal
 //lint:file-ignore ST1005 errors here can be capitalized
 
 import (
+	"errors"
 	"fmt"
 	"io"
 	"net/rpc"
@@ -242,7 +243,6 @@ func (t *Term) sigintGuard(ch <-chan os.Signal, multiClient bool) {
 			default:
 				fmt.Fprintln(t.stdout, "only p or q allowed")
 			}
-
 		} else {
 			fmt.Fprintf(t.stdout, "received SIGINT, stopping process (will not forward signal)\n")
 			_, err := t.client.Halt()
@@ -266,7 +266,7 @@ func (t *Term) Run() (int, error) {
 
 	fns := trie.New()
 	cmds := trie.New()
-	funcs, _ := t.client.ListFunctions("")
+	funcs, _ := t.client.ListFunctions("", 0)
 	for _, fn := range funcs {
 		fns.Add(fn, nil)
 	}
@@ -360,7 +360,7 @@ func (t *Term) Run() (int, error) {
 				fmt.Fprintln(t.stdout, "exit")
 				return t.handleExit()
 			}
-			return 1, fmt.Errorf("Prompt for input failed.\n")
+			return 1, errors.New("Prompt for input failed.\n")
 		}
 		t.stdout.Echo(t.prompt + cmdstr + "\n")
 
