@@ -1,5 +1,4 @@
 //go:build !windows
-// +build !windows
 
 package debugger
 
@@ -20,10 +19,6 @@ import (
 )
 
 func TestDebugger_LaunchNoExecutablePerm(t *testing.T) {
-	defer func() {
-		os.Setenv("GOOS", runtime.GOOS)
-		os.Setenv("GOARCH", runtime.GOARCH)
-	}()
 	fixturesDir := protest.FindFixturesDir()
 	buildtestdir := filepath.Join(fixturesDir, "buildtest")
 	debugname := "debug"
@@ -34,9 +29,15 @@ func TestDebugger_LaunchNoExecutablePerm(t *testing.T) {
 		"linux":   "windows",
 	}
 	if runtime.GOARCH == "arm64" && runtime.GOOS == "linux" {
-		os.Setenv("GOARCH", "amd64")
+		t.Setenv("GOARCH", "amd64")
 	}
-	os.Setenv("GOOS", switchOS[runtime.GOOS])
+	if runtime.GOARCH == "ppc64le" && runtime.GOOS == "linux" {
+		t.Setenv("GOARCH", "amd64")
+	}
+	if runtime.GOARCH == "riscv64" && runtime.GOOS == "linux" {
+		t.Setenv("GOARCH", "amd64")
+	}
+	t.Setenv("GOOS", switchOS[runtime.GOOS])
 	exepath := filepath.Join(buildtestdir, debugname)
 	defer os.Remove(exepath)
 	if err := gobuild.GoBuild(debugname, []string{buildtestdir}, fmt.Sprintf("-o %s", exepath)); err != nil {
@@ -51,7 +52,7 @@ func TestDebugger_LaunchNoExecutablePerm(t *testing.T) {
 		t.Fatalf("expected error but none was generated")
 	}
 	if err != api.ErrNotExecutable {
-		t.Fatalf("expected error \"%s\" got \"%v\"", api.ErrNotExecutable, err)
+		t.Fatalf("expected error %q got \"%v\"", api.ErrNotExecutable, err)
 	}
 }
 
@@ -62,8 +63,8 @@ func TestDebugger_LaunchWithTTY(t *testing.T) {
 		}
 	}
 	// Ensure no env meddling is leftover from previous tests.
-	os.Setenv("GOOS", runtime.GOOS)
-	os.Setenv("GOARCH", runtime.GOARCH)
+	t.Setenv("GOOS", runtime.GOOS)
+	t.Setenv("GOARCH", runtime.GOARCH)
 
 	p, tty, err := pty.Open()
 	if err != nil {
